@@ -1,4 +1,4 @@
-from os.path import join
+from os.path import join, exists
 from typing import Tuple
 
 import click
@@ -7,10 +7,11 @@ from multiprocessing import cpu_count
 
 import torch
 
+from mlcomp.utils.io import yaml_load, yaml_dump
 from mlcomp.contrib.search.grid import grid_cells
 from mlcomp.migration.manage import migrate as _migrate
 from mlcomp import ROOT_FOLDER, IP, PORT, \
-    WORKER_INDEX, SYNC_WITH_THIS_COMPUTER, CAN_PROCESS_TASKS
+    WORKER_INDEX, SYNC_WITH_THIS_COMPUTER, CAN_PROCESS_TASKS, CONFIG_FOLDER
 from mlcomp.db.core import Session
 from mlcomp.db.enums import DagType, ComponentType, TaskStatus
 from mlcomp.db.models import Computer
@@ -19,12 +20,14 @@ from mlcomp.db.providers import \
     TaskProvider, \
     StepProvider, \
     ProjectProvider
+from mlcomp.report import create_report, check_statuses
 from mlcomp.utils.config import merge_dicts_smart, dict_from_list_str
-from mlcomp.utils.io import yaml_load, yaml_dump
 from mlcomp.utils.logging import create_logger
-from mlcomp.worker.sync import sync_directed
+from mlcomp.worker.executors.kaggle import Submit
+from mlcomp.worker.sync import sync_directed, correct_folders
 from mlcomp.worker.tasks import execute_by_id
-from mlcomp.utils.misc import memory, disk, get_username
+from mlcomp.utils.misc import memory, disk, get_username, \
+    get_default_network_interface, now
 from mlcomp.server.back.create_dags import dag_standard, dag_pipe
 
 _session = Session.create_session(key=__name__)
@@ -105,14 +108,21 @@ def migrate():
 @click.option('--control_reqs', type=bool, default=True)
 @click.option('--params', multiple=True)
 def dag(config: str, control_reqs: bool, params):
+    check_statuses()
     _dag(config, control_reqs=control_reqs, params=params)
 
 
+@main.command()
+def report():
+    create_report()
+
+
 @main.command()
 @click.argument('config')
 @click.option('--debug', type=bool, default=True)
 @click.option('--params', multiple=True)
 def execute(config: str, debug: bool, params):
+    check_statuses()
     _create_computer()
 
     # Fail all InProgress Tasks
@@ -158,39 +168,75 @@ def execute(config: str, debug: bool, params):
     is_flag=True,
     help='only copy files from all the others to the computer'
 )
-def sync(project: str, computer: str, only_from: bool, only_to: bool):
+@click.option(
+    '--online',
+    is_flag=True,
+    help='sync with only online computers'
+)
+def sync(project: str, computer: str, only_from: bool, only_to: bool,
+         online: bool):
+    """
+    Syncs specified project on this computer with other computers
+    """
+    check_statuses()
     _create_computer()
 
     computer = computer or socket.gethostname()
     provider = ComputerProvider(_session)
     project_provider = ProjectProvider(_session)
     computer = provider.by_name(computer)
-    computers = provider.all()
-    folders_excluded = []
+    computers = provider.all_with_last_activtiy()
     p = project_provider.by_name(project)
     assert p, f'Project={project} is not found'
 
-    ignore = yaml_load(p.ignore_folders)
-    excluded = []
-    for f in ignore:
-        excluded.append(str(f))
+    sync_folders = yaml_load(p.sync_folders)
+    ignore_folders = yaml_load(p.ignore_folders)
 
-    folders_excluded.append([join('data', p.name), excluded])
-    folders_excluded.append([join('models', p.name), []])
+    sync_folders = correct_folders(sync_folders, p.name)
+    ignore_folders = correct_folders(ignore_folders, p.name)
+
+    if not isinstance(sync_folders, list):
+        sync_folders = []
+    if not isinstance(ignore_folders, list):
+        ignore_folders = []
+
+    folders = [[s, ignore_folders] for s in sync_folders]
 
     for c in computers:
         if c.name != computer.name:
+            if online and (now() - c.last_activity).total_seconds() > 100:
+                continue
+
             if not only_from:
-                sync_directed(_session, computer, c, folders_excluded)
+                sync_directed(_session, computer, c, folders)
             if not only_to:
-                sync_directed(_session, c, computer, folders_excluded)
+                sync_directed(_session, c, computer, folders)
 
 
 @main.command()
 def init():
-    # already done by importing mlcomp
-    # that is needed to import it
-    pass
+    env_path = join(CONFIG_FOLDER, '.env')
+    lines = open(env_path).readlines()
+    for i in range(len(lines)):
+        if 'NCCL_SOCKET_IFNAME' in lines[i]:
+            interface = get_default_network_interface()
+            if interface:
+                lines[i] = f'NCCL_SOCKET_IFNAME={interface}\n'
+    open(env_path, 'w').writelines(lines)
+
+
+@main.command()
+def submit():
+    assert exists('submit.yml'), 'no file submit.yml'
+
+    data = yaml_load(file='submit.yml')
+    submit = Submit(
+        competition=data['competition'],
+        submit_type='kernel',
+        max_size=data['max_size'],
+        folders=data['folders']
+    )
+    submit.work()
 
 
 if __name__ == '__main__':
